@@ -31,7 +31,7 @@
 #'     \item{\code{\link{autostrat}}} {to automatically evaluate and rank
 #'     top-performing strategies.}
 #'     \item{\code{\link{mlgrid}}} {to generate a numeric representation of the
-#'     relationship between ichimoku cloud chart elements.}
+#'     ichimoku cloud chart.}
 #'     \item{\code{\link{relative}}} {to produce a statistical summary of the
 #'     latest ichimoku numeric representation relative to historical values.}
 #' }
@@ -45,6 +45,8 @@
 #'     the latest live data stream / quote from the OANDA fxTrade API.}
 #'     \item{\code{\link{oanda_view}}} {for a market overview showing the
 #'     relative performance of constituents.}
+#'     \item{\code{\link{oanda_orders}} / \code{\link{oanda_positions}}} {to
+#'     retrieve the aggregate OANDA fxTrade order / position book.}
 #' }
 #'
 #' @encoding UTF-8
@@ -61,8 +63,7 @@
 #'     \url{https://CRAN.R-project.org/package=ichimoku}.
 #'
 #' @useDynLib ichimoku, .registration = TRUE
-#' @importFrom curl curl_fetch_memory curl_fetch_stream handle_setheaders
-#'     new_handle
+#' @importFrom curl curl curl_fetch_memory handle_setheaders new_handle
 #' @importFrom ggplot2 aes autoplot coord_flip element_blank element_line
 #'     element_rect element_text geom_col geom_line geom_rect geom_ribbon
 #'     geom_segment geom_vline ggplot ggplotGrob guides labs margin rel
@@ -70,8 +71,9 @@
 #'     scale_y_continuous theme theme_grey %+replace%
 #' @importFrom grid grid.draw grid.newpage unit
 #' @importFrom gtable gtable gtable_add_grob
-#' @importFrom jsonlite parse_json
-#' @importFrom stats na.omit sd setNames
+#' @importFrom jsonlite parse_json stream_in
+#' @importFrom stats na.omit sd
+#' @importFrom tibble as_tibble tbl_sum
 #' @importFrom utils str
 #' @importFrom xts endpoints
 #' @importFrom zoo coredata index
@@ -83,8 +85,8 @@ NULL
 utils::globalVariables(".data")
 
 .onLoad <- function(libname, pkgname) {
-  do_oanda <- do_oanda()
-  do_oanda <<- do_oanda
+  do_ <- do_()
+  do_ <<- do_
   invisible()
 }
 
