@@ -11,13 +11,13 @@ test_that("extraplot ok", {
   expect_s3_class(extraplot(strat, theme = "original", type = "r", custom = "nonexist"), "gtable")
   expect_s3_class(extraplot(strat, type = "line", custom = "posn"), "gtable")
   expect_s3_class(extraplot(cloud, type = "bar", custom = "vol"), "gtable")
-  expect_warning(expect_s3_class(extraplot(cloud), "ggplot"), "'type' not specified")
+  expect_warning(expect_s3_class(extraplot(cloud, theme = "fresh"), "ggplot"), "'type' not specified")
   expect_warning(expect_s3_class(extraplot(cloud, type = "line"), "ggplot"), "'custom' not specified")
   expect_warning(expect_s3_class(extraplot(cloud, type = "bar", custom = "vix"), "ggplot"), "does not match")
 })
 
 test_that("ichimoku plot method ok", {
   expect_s3_class(expect_invisible(plot(cloud, window = "2020-06", subtitle = "subtitle")), "ichimoku")
-  expect_s3_class(expect_invisible(plot(strat, type = "s", theme = "dark", strat = FALSE)), "ichimoku")
-  expect_s3_class(expect_invisible(plot(cloud, custom = "close")), "ichimoku")
+  expect_s3_class(expect_invisible(plot(strat, theme = "dark", type = "s", strat = FALSE)), "ichimoku")
+  expect_s3_class(expect_invisible(plot(cloud, theme = "conceptual", custom = "close")), "ichimoku")
 })
